from abc import ABC, abstractmethod
from typing import Optional

from cpl_query.extension import List

from bot_api.filter.auth_user_select_criteria import AuthUserSelectCriteria
from bot_api.model.auth_user_dto import AuthUserDTO
from bot_api.model.auth_user_filtered_result_dto import AuthUserFilteredResultDTO
from bot_api.model.email_string_dto import EMailStringDTO
from bot_api.model.o_auth_dto import OAuthDTO
from bot_api.model.reset_password_dto import ResetPasswordDTO
from bot_api.model.token_dto import TokenDTO
from bot_api.model.update_auth_user_dto import UpdateAuthUserDTO
from bot_data.model.auth_user import AuthUser


class AuthServiceABC(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def generate_token(self, user: AuthUser) -> str:
        pass

    @abstractmethod
    def decode_token(self, token: str) -> dict:
        pass

    @abstractmethod
    def get_decoded_token_from_request(self) -> dict:
        pass

    @abstractmethod
    def find_decoded_token_from_request(self) -> Optional[dict]:
        pass

    @abstractmethod
    async def get_all_auth_users_async(self) -> List[AuthUserDTO]:
        pass

    @abstractmethod
    async def get_filtered_auth_users_async(self, criteria: AuthUserSelectCriteria) -> AuthUserFilteredResultDTO:
        pass

    @abstractmethod
    async def get_auth_user_by_email_async(self, email: str, with_password: bool = False) -> AuthUserDTO:
        pass

    @abstractmethod
    async def find_auth_user_by_email_async(self, email: str) -> AuthUserDTO:
        pass

    @abstractmethod
    async def add_auth_user_async(self, user_dto: AuthUserDTO):
        pass

    @abstractmethod
    async def add_auth_user_by_oauth_async(self, dto: OAuthDTO):
        pass

    @abstractmethod
    async def add_auth_user_by_discord_async(self, user_dto: AuthUserDTO, dc_id: int) -> OAuthDTO:
        pass

    @abstractmethod
    async def update_user_async(self, update_user_dto: UpdateAuthUserDTO):
        pass

    @abstractmethod
    async def update_user_as_admin_async(self, update_user_dto: UpdateAuthUserDTO):
        pass

    @abstractmethod
    async def delete_auth_user_by_email_async(self, email: str):
        pass

    @abstractmethod
    async def delete_auth_user_async(self, user_dto: AuthUserDTO):
        pass

    @abstractmethod
    async def verify_login(self, token_str: str) -> bool:
        pass

    @abstractmethod
    async def login_async(self, user_dto: AuthUserDTO) -> TokenDTO:
        pass

    @abstractmethod
    async def login_discord_async(self, oauth_dto: AuthUserDTO) -> TokenDTO:
        pass

    @abstractmethod
    async def refresh_async(self, token_dto: TokenDTO) -> TokenDTO:
        pass

    @abstractmethod
    async def revoke_async(self, token_dto: TokenDTO):
        pass

    @abstractmethod
    async def confirm_email_async(self, id: str) -> bool:
        pass

    @abstractmethod
    async def forgot_password_async(self, email: str):
        pass

    @abstractmethod
    async def confirm_forgot_password_async(self, id: str) -> EMailStringDTO:
        pass

    @abstractmethod
    async def reset_password_async(self, rp_dto: ResetPasswordDTO):
        pass
