from cpl_core.application import ApplicationExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.configuration.authentication_settings import AuthenticationSettings
from bot_api.route.route import Route
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC


class AppApiExtension(ApplicationExtensionABC):
    def __init__(self):
        ApplicationExtensionABC.__init__(self)

    async def run(self, config: ConfigurationABC, services: ServiceProviderABC):
        feature_flags: FeatureFlagsSettings = config.get_configuration(FeatureFlagsSettings)
        if not feature_flags.get_flag(FeatureFlagsEnum.api_module):
            return

        auth_settings: AuthenticationSettings = config.get_configuration(AuthenticationSettings)
        auth_users: AuthUserRepositoryABC = services.get_service(AuthUserRepositoryABC)
        auth: AuthServiceABC = services.get_service(AuthServiceABC)
        Route.init_authorize(auth_users, auth)
