from ariadne import graphql_sync
from ariadne.constants import PLAYGROUND_HTML
from cpl_core.configuration import ConfigurationABC
from cpl_core.environment import ApplicationEnvironmentABC
from flask import request, jsonify

from bot_api.logging.api_logger import ApiLogger
from bot_api.route.route import Route
from bot_graphql.schema import Schema


class GraphQLController:
    BasePath = f"/api/graphql"

    def __init__(
        self,
        config: ConfigurationABC,
        env: ApplicationEnvironmentABC,
        logger: ApiLogger,
        schema: Schema,
    ):
        self._config = config
        self._env = env
        self._logger = logger
        self._schema = schema

    @Route.get(f"{BasePath}/playground")
    async def playground(self):
        return PLAYGROUND_HTML, 200

    @Route.post(f"{BasePath}")
    async def graphql(self):
        data = request.get_json()

        # Note: Passing the request to the context is optional.
        # In Flask, the current request is always accessible as flask.request
        success, result = graphql_sync(self._schema.schema, data, context_value=request)

        return jsonify(result), 200 if success else 400
