from abc import ABC, abstractmethod
from typing import Optional

from cpl_query.extension import List

from bot_data.model.statistic import Statistic


class StatisticRepositoryABC(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def get_statistics(self) -> List[Statistic]:
        pass

    @abstractmethod
    def get_statistics_by_server_id(self, server_id: int) -> List[Statistic]:
        pass

    @abstractmethod
    def get_statistic_by_id(self, id: int) -> Statistic:
        pass

    @abstractmethod
    def get_statistic_by_name(self, name: str, server_id: int) -> Statistic:
        pass

    @abstractmethod
    def find_statistic_by_name(self, name: str, server_id: int) -> Optional[Statistic]:
        pass

    @abstractmethod
    def add_statistic(self, statistic: Statistic):
        pass

    @abstractmethod
    def update_statistic(self, statistic: Statistic):
        pass

    @abstractmethod
    def delete_statistic(self, statistic: Statistic):
        pass
