import asyncio

import discord
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_core.service.client_utils_service import ClientUtilsService


class MassMoveCommand(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        translate: TranslatePipe,
        client_utils: ClientUtilsService,
    ):
        DiscordCommandABC.__init__(self)
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._t = translate
        self._client_utils = client_utils

    @commands.hybrid_command(name="mass-move")
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def mass_move(
        self,
        ctx: Context,
        channel_to: discord.VoiceChannel,
        channel_from: discord.VoiceChannel = None,
    ):
        self._logger.debug(__name__, f"Received command mass-move {ctx}")

        if channel_from is None and ctx.author.voice is None:
            await self._message_service.send_ctx_msg(
                ctx, self._t.transform("modules.base.mass_move.channel_from_error")
            )
            return

        if channel_from is None:
            channel_from = ctx.author.voice.channel

        moves = [member.move_to(channel_to) for member in channel_from.members]
        move_count = len(moves)
        await asyncio.gather(*moves)
        self._client_utils.moved_users(ctx.guild.id, move_count)

        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.base.mass_move.moved").format(channel_from.mention, channel_to.mention),
        )

        self._logger.trace(__name__, f"Finished mass-move command")
